// Hitchin Hackspace Arduino Starter Kit
// https://hackhitchin.org.uk/arduinokit/
// https://wiki.hackhitchin.org.uk/index.php?title=Arduino_Kit#Project:_Counter
//
// Code Examples
// "7 Segment Display - Counter"

// The display segments are named as follows:
//  aaa
// f   b
// f   b
// f   b
//  ggg
// e   c
// e   c
// e   c
//  ddd

// Declare some variables representing the segments.
// 'const' means the value of these variables should be constant - they can't be reassigned.
const byte SEG_G = 0b1000000;
const byte SEG_F = 0b0100000;
const byte SEG_E = 0b0010000;
const byte SEG_D = 0b0001000;
const byte SEG_C = 0b0000100;
const byte SEG_B = 0b0000010;
const byte SEG_A = 0b0000001;

// This creates an array of ten elements - one for each digit '0' to '9'
// containing the patterns to display each.
const byte digits[10] = {
  // The '|' symbol combines the patterns together by performing a bitwise 'or'
  // Zero has all segments lit except G.
  SEG_F | SEG_E | SEG_D | SEG_C | SEG_B | SEG_A,
  // One has segments B and C lit.
  SEG_B | SEG_C, 
  // Two has A, B, G, E and D lit.
  SEG_A | SEG_B | SEG_G | SEG_E | SEG_D,
  // Three has A, B, G, C and D lit.
  SEG_A | SEG_B | SEG_G | SEG_C | SEG_D,
  // Four has F, G, B and C lit.
  SEG_F | SEG_G | SEG_B | SEG_C,
  // Five has A, G, F, D and C lit.
  SEG_A | SEG_G | SEG_F | SEG_D | SEG_C,
  // Six has all segments lit except B.
  SEG_G | SEG_F | SEG_E | SEG_D | SEG_C | SEG_A,
  // Seven has A, B and C lit.
  SEG_A | SEG_B | SEG_C,
  // Eight has all segments lit.
  SEG_G | SEG_F | SEG_E | SEG_D | SEG_C | SEG_B | SEG_A,
  // Nine has all segments lit except E.
  SEG_G | SEG_F | SEG_D | SEG_C | SEG_B | SEG_A,
};

void setup() {
  // Digital pins 2 to 8 are used to connect to the display.
  for (int i = 2; i <= 8; i = i + 1) {
    pinMode(i, OUTPUT);
  }
}

// Create a counter variable.
int counter = 0;

void loop() {
  // Retrieve from the array the pattern to display the current digit.
  byte pattern = digits[counter];

  // If the pattern includes segment G, light it. Otherwise, turn it off.
  if ((pattern & SEG_G) != 0) {
    digitalWrite(8, HIGH);
  }
  else {
    digitalWrite(8, LOW);
  }
  
  // Similarly for the remaining segments
  if ((pattern & SEG_F) != 0) {
    digitalWrite(7, HIGH);
  }
  else {
    digitalWrite(7, LOW);
  }

  if ((pattern & SEG_E) != 0) {
    digitalWrite(6, HIGH);
  }
  else {
    digitalWrite(6, LOW);
  }

  if ((pattern & SEG_D) != 0) {
    digitalWrite(5, HIGH);
  }
  else {
    digitalWrite(5, LOW);
  }

  if ((pattern & SEG_C) != 0) {
    digitalWrite(4, HIGH);
  }
  else {
    digitalWrite(4, LOW);
  }

  if ((pattern & SEG_B) != 0) {
    digitalWrite(3, HIGH);
  }
  else {
    digitalWrite(3, LOW);
  }

  if ((pattern & SEG_A) != 0) {
    digitalWrite(2, HIGH);
  }
  else {
    digitalWrite(2, LOW);
  }

  // Advance the counter.
  counter = counter + 1;

  // Limit it to '9'.
  if (counter == 10) {
    counter = 0;
  } 

  // Delay one second.
  delay(1000);
}
