
// Hitchin Hackspace Arduino Starter Kit
// https://hackhitchin.org.uk/arduinokit/
// https://wiki.hackhitchin.org.uk/index.php?title=Arduino_Kit#Project:_Dice
//
// Code Examples
// "Dice"

// The display segments are named as follows:
//  aaa
// f   b
// f   b
// f   b
//  ggg
// e   c
// e   c
// e   c
//  ddd


// Declare some variables representing the segments.
// 'const' means the value of these variables should be constant - they can't be reassigned.
const byte SEG_G = 0b1000000;
const byte SEG_F = 0b0100000;
const byte SEG_E = 0b0010000;
const byte SEG_D = 0b0001000;
const byte SEG_C = 0b0000100;
const byte SEG_B = 0b0000010;
const byte SEG_A = 0b0000001;

// This creates an array of ten elements - one for each digit '0' to '9'
// containing the patterns to display each.
const byte digits[10] = {
  // The '|' symbol combines the patterns together by performing a bitwise 'or'
  // Zero has all segments lit except G.
  SEG_F | SEG_E | SEG_D | SEG_C | SEG_B | SEG_A,
  // One has segments B and C lit.
  SEG_B | SEG_C, 
  // Two has A, B, G, E and D lit.
  SEG_A | SEG_B | SEG_G | SEG_E | SEG_D,
  // Three has A, B, G, C and D lit.
  SEG_A | SEG_B | SEG_G | SEG_C | SEG_D,
  // Four has F, G, B and C lit.
  SEG_F | SEG_G | SEG_B | SEG_C,
  // Five has A, G, F, D and C lit.
  SEG_A | SEG_G | SEG_F | SEG_D | SEG_C,
  // Six has all segments lit except B.
  SEG_G | SEG_F | SEG_E | SEG_D | SEG_C | SEG_A,
  // Seven has A, B and C lit.
  SEG_A | SEG_B | SEG_C,
  // Eight has all segments lit.
  SEG_G | SEG_F | SEG_E | SEG_D | SEG_C | SEG_B | SEG_A,
  // Nine has all segments lit except E.
  SEG_G | SEG_F | SEG_D | SEG_C | SEG_B | SEG_A,
};

void setup() {
  // Digital pins 2 to 8 are used to connect to the display.
  for (int i = 2; i <= 8; i = i + 1) {
    pinMode(i, OUTPUT);
  }
  // Digital pin 9 is the button input.
  pinMode(9, INPUT);
}



// Create a place to store the result of the roll.
int dicevalue = 0;

// This function takes a bit pattern as an argument and shows it on the display.
void displayPattern(byte pattern) {
  // Iterate over every segment from A to G.
  for (int segment = 0; segment < 7; segment = segment + 1) {
    // The bit pattern for this segment is:
    byte segpattern = 0b0000001 << segment;
    // The pin for this segment is:
    int pin = 2 + segment;
    // If the pattern we want to display includes includes this segment,
    if ((pattern & segpattern) != 0) {
      // Light it.
      digitalWrite(pin, HIGH);
    }
    else {
      // Otherwise, extinguish it.
      digitalWrite(pin, LOW);
    }
  }  
}

// This function takes a digit as an argument and shows it on the display.
void displayDigit(int value) {
  // Retrieve from the array the pattern to display the current digit.
  byte pattern = digits[value];

  displayPattern(pattern);
}

void loop() {
  displayDigit(dicevalue);
  
  // Is the button being held?
  if (digitalRead(9)) {
    // Roll the dice!
    // In the random() function, the first value can be generated but the second one can't.
    dicevalue = random(1, 7);

    // Delay just a bit, so the number change is visible.
    delay(100);
  }
}
